@@ -454,9 +454,9 @@ do_primary_unregister(void)
 
 			/*
 			 * This appears to be the cluster primary - cowardly refuse to
-			 * delete the record
+			 * delete the record, unless --force is supplied.
 			 */
-			if (primary_node_info.node_id == target_node_info_ptr->node_id)
+			if (primary_node_info.node_id == target_node_info_ptr->node_id && !runtime_options.force)
 			{
 				log_error(_("node \"%s\" (ID: %i) is the current primary node, unable to unregister"),
 						  target_node_info_ptr->node_name,
@@ -575,4 +575,5 @@ do_primary_help(void)
 
 	puts("");
 
+	printf(_("%s home page: <%s>\n"), "repmgr", REPMGR_URL);
 }
