@@ -17,13 +17,12 @@ extern volatile sig_atomic_t got_SIGHUP;
 extern MonitoringState monitoring_state;
 extern instr_time degraded_monitoring_start;
 
-extern t_configuration_options config_file_options;
 extern t_node_info local_node_info;
 extern PGconn *local_conn;
 extern bool startup_event_logged;
 extern char pid_file[MAXPGPATH];
 
-bool		check_upstream_connection(PGconn **conn, const char *conninfo);
+bool		check_upstream_connection(PGconn **conn, const char *conninfo, PGconn **paired_conn);
 void		try_reconnect(PGconn **conn, t_node_info *node_info);
 
 int			calculate_elapsed(instr_time start_time);
